# frozen_string_literal: true

require "is/stateful"

require_relative "tenet"

module Doctrine
  class Definition
    class << self
      def subject(value = value_omitted = true, &block)
        if block
          @subject = block
          @subject_type = :block
        elsif !value_omitted
          @subject = value
          @subject_type = :value
        end
      end

      def tenet(description, &block)
        tenets << Tenet.new(description: description, &block)
      end
    end

    include Is::Stateful
    state :tenets, :class, default: []

    # [public]
    #
    def subject(value = value_omitted = true, &block)
      if block
        @subject = block
        @subject_type = :block
      elsif !value_omitted
        @subject = value
        @subject_type = :value
      end
    end

    # [public]
    #
    def assert
      subject, subject_type = find_subject_and_subject_type

      if subject_type
        unless yield(resolve(subject, subject_type))
          raise AssertionFailed
        end
      else
        raise UndefinedSubject
      end
    end

    # [public]
    #
    def capture
      yield
    rescue => error
      @subject_type = :error
      @subject = error
    end

    private def find_subject_and_subject_type
      if @subject_type
        [@subject, @subject_type]
      elsif (subject_type = self.class.instance_variable_get(:@subject_type))
        [self.class.instance_variable_get(:@subject), subject_type]
      end
    end

    private def resolve(subject, subject_type)
      case subject_type
      when :block
        subject.call
      when :error, :value
        subject
      end
    end
  end
end
