# frozen_string_literal: true

require_relative "worker"

module Doctrine
  # [public]
  #
  class Runner
    include MonitorMixin

    def initialize(reporter:)
      @reporter = reporter
      @condition = new_cond
      @queue = Queue.new
      @worker = Worker.new(queue: @queue, runner: self)
      @thread = Thread.new { @worker.run }
      @enqueued = []
      @working = []
      @finished = []
    end

    # [public]
    #
    def run(tenet)
      @enqueued << tenet
      @queue << tenet
    end

    # [public]
    #
    def starting(tenet)
      @reporter.starting(tenet)
      @enqueued.delete(tenet)
      @working << tenet
    end

    # [public]
    #
    def finished(tenet)
      @reporter.finished(tenet)
      @working.delete(tenet)
      @finished << tenet

      synchronize do
        @condition.signal
      end
    end

    # [public]
    #
    def wait
      synchronize do
        @condition.wait_until do
          @enqueued.empty? && @working.empty?
        end
      end
    end
  end
end
