# frozen_string_literal: true

require "is/definable"
require "is/factory"
require "is/loadable"
require "is/stateful"

require_relative "reporter"
require_relative "runner"
require_relative "run"

module Doctrine
  # [public]
  #
  class Suite
    include Is::Definable
    include Is::Factory
    include Is::Loadable
    include Is::Stateful

    state :doctrines, default: []

    # [public]
    #
    def self.run
      reporter = Reporter.new
      runner = Runner.new(reporter: reporter)

      runs = []
      @doctrines.each do |doctrine|
        doctrine.tenets.each do |tenet|
          run = Run.new(doctrine: doctrine, tenet: tenet)
          runner.run(run)
          runs << run
        end
      end

      runner.wait

      reporter.summarize(runs)
    end
  end
end
